@@ -974,10 +974,18 @@ static int query(struct dns_query **qp, struct dnsc *dnsc, uint8_t opcode,
 	struct dns_query *q = NULL;
 	struct dnshdr hdr;
 	int err = 0;
+	bool use_getaddrinfo = false;
+	bool srv_available = srvv && srvc && *srvc != 0;
 
-	if (!dnsc || !name || !srvv || !srvc || !(*srvc))
+	if (!dnsc || !name)
 		return EINVAL;
 
+	use_getaddrinfo = dnsc->conf.getaddrinfo &&
+		(type == DNS_TYPE_A || type == DNS_TYPE_AAAA);
+
+	if (!srv_available && !use_getaddrinfo)
+		return ENOTSUP;
+
 	if (DNS_QTYPE_AXFR == type)
 		proto = IPPROTO_TCP;
 
@@ -1029,8 +1037,7 @@ static int query(struct dns_query **qp, struct dnsc *dnsc, uint8_t opcode,
 		list_init(q->rrlv[i]);
 	}
 
-	if (dnsc->conf.getaddrinfo &&
-	    (q->type == DNS_TYPE_A || q->type == DNS_TYPE_AAAA)) {
+	if (use_getaddrinfo) {
 		err = query_getaddrinfo(q);
 		if (err)
 			goto error;
@@ -1217,9 +1224,7 @@ int dnsc_alloc(struct dnsc **dcpp, const struct dnsc_conf *conf,
 {
 	struct dnsc *dnsc;
 	struct sa laddr;
-#ifdef HAVE_INET6
 	struct sa laddr6;
-#endif
 	int err;
 
 	if (!dcpp)
@@ -1241,10 +1246,9 @@ int dnsc_alloc(struct dnsc **dcpp, const struct dnsc_conf *conf,
 	sa_set_str(&laddr, "0.0.0.0", 0);
 	err  = udp_listen(&dnsc->us, &laddr, udp_recv_handler, dnsc);
 
-#ifdef HAVE_INET6
 	sa_set_str(&laddr6, "::", 0);
 	err &= udp_listen(&dnsc->us6, &laddr6, udp_recv_handler, dnsc);
-#endif
+
 	if (err)
 		goto out;
 
@@ -1404,3 +1408,20 @@ bool dnsc_getaddrinfo_enabled(struct dnsc *dnsc)
 
 	return dnsc->conf.getaddrinfo;
 }
+
+
+/**
+ * Return if getaddrinfo usage is enabled and exclusive,
+ * i.e. there are no DNS servers known explicitly
+ *
+ * @param dnsc  DNS Client
+ *
+ * @return true if DNS servers are available, false otherwise
+ */
+bool dnsc_getaddrinfo_only(const struct dnsc *dnsc)
+{
+	if (!dnsc)
+		return false;
+
+	return dnsc->conf.getaddrinfo && dnsc->srvc == 0;
+}
